package com.mustafasha.simbadan;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import androidx.core.content.ContextCompat;
import android.Manifest;
import java.util.ArrayList;
import android.content.pm.PackageManager;
import androidx.core.app.ActivityCompat;
import android.content.Intent;
import android.net.Uri;
import android.telecom.TelecomManager;
import android.content.Context;
import java.util.List;
import android.telecom.PhoneAccountHandle;
import android.telephony.SubscriptionInfo;
import android.telephony.SubscriptionManager;

/** SimbadanPlugin */
public class SimbadanPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "simbadan");
    channel.setMethodCallHandler(this);
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "simbadan");
    channel.setMethodCallHandler(new SimbadanPlugin());
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("dialBySim")) {
      String numberToDial = call.argument("numberToDial");
      Integer simSlot = call.argument("simSlot");

      callNumberBySimSlot(numberToDial, simSlot);

      result.success(null);
    } else if (call.method.equals("dialByCarrier")) {
      String numberToDial = call.argument("numberToDial");
      String carrier = call.argument("carrier");

      callNumberByCarrierName(numberToDial, carrier);

      result.success(null);
  } else if (call.method.equals("getCarriers")) {
    result.success(getCarrierNames());
  } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  public void callNumberByCarrierName(String number, String carrierName) {
    int slot = getSimSlotByCarrierName(carrierName);

    callNumberBySimSlot(number, slot);
  }

  private int getSimSlotByCarrierName(String carrierName){
    int simSlot = 0;
    
    List<SubscriptionInfo> subscriptionInfos = SubscriptionManager.from(this.getContext()).getActiveSubscriptionInfoList();

    for(int i=0; i<subscriptionInfos.size();i++)
    {
        SubscriptionInfo lsuSubscriptionInfo = subscriptionInfos.get(i);
        String simCarrierName = lsuSubscriptionInfo.getCarrierName().toString();

        if(carrierName.toLowerCase().indexOf(simCarrierName.toLowerCase()) > -1){
          simSlot = i;
          break;
        }
    }

    return simSlot;
  }

  private ArrayList<String> getCarrierNames(){
    if(!checkPermissions()) return new ArrayList<String>();

    ArrayList<String> carrierNames = new ArrayList<String>();
    List<SubscriptionInfo> subscriptionInfos = SubscriptionManager.from(this.getContext()).getActiveSubscriptionInfoList();

    for(int i=0; i<subscriptionInfos.size();i++)
    {
        SubscriptionInfo lsuSubscriptionInfo = subscriptionInfos.get(i);
        String simCarrierName = lsuSubscriptionInfo.getCarrierName().toString();

        carrierNames.add(simCarrierName);
    }

    return carrierNames;
  }

  public void callNumberBySimSlot(String number, int slot) {

    if(!checkPermissions()) return;

    Intent callIntent = createCallIntent(number, slot);

    for (String s : simSlotName){
        callIntent.putExtra(s,slot);
    }

    this.getContext().startActivity(callIntent);
  }

  private boolean checkPermissions(){
    int callPhone = ContextCompat.checkSelfPermission(this.getContext().getApplicationContext(), Manifest.permission.CALL_PHONE);
    int readPhoneState = ContextCompat.checkSelfPermission(this.getContext().getApplicationContext(), Manifest.permission.READ_PHONE_STATE);

    ArrayList<String> permiss = new ArrayList<String>();
    if (callPhone != PackageManager.PERMISSION_GRANTED) {
        permiss.add(Manifest.permission.CALL_PHONE);
    }

    if (readPhoneState != PackageManager.PERMISSION_GRANTED) {
        permiss.add(Manifest.permission.READ_PHONE_STATE);
    }

    if (permiss.size() > 0) {
        String[] per = new String[permiss.size()];
        ActivityCompat.requestPermissions(this, permiss.toArray(per), REQUEST_CODE_PERMISSION);
        return false;
    } else {
        return true;
    }
  }

  private Intent createCallIntent(String number, int slot){
    Intent callIntent = new Intent(Intent.ACTION_CALL);
    callIntent.setData(Uri.parse("tel:" + Uri.encode(number)));
    callIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    callIntent.putExtra("com.android.phone.force.slot", true);

    if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
        callIntent.putExtra("Cdma_Supp", true);
        TelecomManager telecomManager = (TelecomManager) this.getContext().getSystemService(Context.TELECOM_SERVICE);
        List<PhoneAccountHandle> phoneAccountHandleList = telecomManager.getCallCapablePhoneAccounts();
        if (phoneAccountHandleList != null && phoneAccountHandleList.size() > 0)
        {
            callIntent.putExtra("android.telecom.extra.PHONE_ACCOUNT_HANDLE",
                    phoneAccountHandleList.get(slot));
        }
    }

    return callIntent;
  }
}
